@@ -14,7 +14,7 @@ sub validate {
     return $self->bad_command("Select a mailbox first")
         unless $self->connection->is_selected;
 
-    return $self->bad_command("Mailbox is read-only") if $self->connection->selected_read_only;
+    return $self->bad_command("Mailbox is read-only") if $self->connection->selected->read_only;
 
     my @options = $self->parsed_options;
     return $self->bad_command("Not enough options") if @options < 3;
