@@ -12,7 +12,18 @@ use Net::IMAP::Server::Error;
 use Net::IMAP::Server::Command;
 
 __PACKAGE__->mk_accessors(
-    qw(server io_handle _selected selected_read_only model pending temporary_messages temporary_sequence_map previous_exists untagged_expunge untagged_fetch ignore_flags last_poll in_poll commands timer coro _session_flags)
+    qw(server coro io_handle model auth
+       timer commands pending
+       selected_read_only
+       _selected
+
+       temporary_messages temporary_sequence_map
+       ignore_flags
+       _session_flags
+
+       last_poll previous_exists in_poll
+       _unsent_expunge _unsent_fetch
+       )
 );
 
 =head1 NAME
@@ -35,13 +46,13 @@ sub new {
     my $class = shift;
     my $self  = $class->SUPER::new(
         {   @_,
-            state            => "unauth",
-            untagged_expunge => [],
-            untagged_fetch   => {},
-            last_poll        => time,
-            commands         => 0,
-            coro             => $Coro::current,
-            _session_flags   => {},
+            state           => "unauth",
+            _unsent_expunge => [],
+            _unsent_fetch   => {},
+            last_poll       => time,
+            commands        => 0,
+            coro            => $Coro::current,
+            _session_flags  => {},
         }
     );
     $self->update_timer;
@@ -81,7 +92,7 @@ sub auth {
     my $self = shift;
     if (@_) {
         $self->{auth} = shift;
-        $self->server->model_class->require || warn $@;
+        $self->server->model_class->require || $self->log(1, $@);
         $self->update_timer;
         $self->model(
             $self->server->model_class->new( { auth => $self->{auth} } ) );
@@ -171,12 +182,12 @@ sub handle_lines {
             cede;
         }
 
-        $self->log(
+        $self->log( 4,
             "-(@{[$self]},@{[$self->auth ? $self->auth->user : '???']},@{[$self->is_selected ? $self->selected->full_path : 'unselected']}): Connection closed by remote host"
         );
     };
     my $err = $@;
-    warn $err
+    $self->log(1, $err)
         if $err and not( $err eq "Error printing\n" or $err eq "Timeout\n" );
     eval { $self->out("* BYE Idle timeout; I fell asleep.") if $err eq "Timeout\n"; };
     $self->close;
@@ -209,6 +220,10 @@ sub update_timer {
 
 Returns the L<EV> watcher in charge of the inactivity timer.
 
+=head2 commands
+
+Returns the number of client commands the connection has processed.
+
 =head2 handle_command
 
 Handles a single line from the client.  This is not quite the same as
@@ -226,8 +241,10 @@ sub handle_command {
     my $self    = shift;
     my $content = shift;
 
-    $self->log(
-        "C(@{[$self]},@{[$self->auth ? $self->auth->user : '???']},@{[$self->is_selected ? $self->selected->full_path : 'unselected']}): $content"
+    my $output = $content;
+    $output =~ s/[\r\n]+$//;
+    $self->log( 4,
+        "C(@{[$self]},@{[$self->auth ? $self->auth->user : '???']},@{[$self->is_selected ? $self->selected->full_path : 'unselected']}): $output"
     );
 
     if ( $self->pending ) {
@@ -258,7 +275,7 @@ sub handle_command {
             $handler->bad_command($1);
         } else {
             $handler->no_command("Server error");
-            $self->log($error);
+            $self->log(1, $error);
         }
     }
 }
@@ -282,7 +299,7 @@ sub class_for {
     $cmd_class->require();
     my $err = $@;
     if ($err and $err !~ /^Can't locate $class_path.pm in \@INC/) {
-        warn $@;
+        $self->log(1, $@);
         $cmd_class = "Net::IMAP::Server::Error";
     }
 
@@ -415,7 +432,10 @@ sub force_poll {
 Gets or sets the last time the selected mailbox was polled, in seconds
 since the epoch.
 
-=cut
+=head2 previous_exists
+
+The high-water mark of how many messages the client has been told are
+in the mailbox.
 
 =head2 send_untagged
 
@@ -440,32 +460,32 @@ sub send_untagged {
         $self->in_poll(0);
     }
 
-    for my $s ( keys %{ $self->untagged_fetch } ) {
+    for my $s ( keys %{ $self->_unsent_fetch } ) {
         my ($m) = $self->get_messages($s);
         $self->untagged_response(
                   $s 
                 . " FETCH "
                 . Net::IMAP::Server::Command->data_out(
-                [ $m->fetch( [ keys %{ $self->untagged_fetch->{$s} } ] ) ]
+                [ $m->fetch( [ keys %{ $self->_unsent_fetch->{$s} } ] ) ]
                 )
         );
     }
-    $self->untagged_fetch( {} );
+    $self->_unsent_fetch( {} );
 
     if ( $args{expunged} ) {
 
 # Make sure that they know of at least the existence of what's being expunged.
         my $max = 0;
-        $max = $max < $_ ? $_ : $max for @{ $self->untagged_expunge };
+        $max = $max < $_ ? $_ : $max for @{ $self->_unsent_expunge };
         $self->untagged_response("$max EXISTS")
             if $max > $self->previous_exists;
 
         # Send the expunges, clear out the temporary message store
         $self->previous_exists(
-            $self->previous_exists - @{ $self->untagged_expunge } );
+            $self->previous_exists - @{ $self->_unsent_expunge } );
         $self->untagged_response( map {"$_ EXPUNGE"}
-                @{ $self->untagged_expunge } );
-        $self->untagged_expunge( [] );
+                @{ $self->_unsent_expunge } );
+        $self->_unsent_expunge( [] );
         $self->temporary_messages(undef);
     }
 
@@ -547,28 +567,15 @@ sub capability {
     return $base;
 }
 
-=head2 session_flags MESSAGE
+=head2 log SEVERITY, MESSAGE
 
-=cut
-
-sub session_flags {
-    my $self = shift;
-    my ($message) = shift;
-    $self->_session_flags->{$message . ""} ||= {};
-    return $self->_session_flags->{$message . ""};
-}
-
-=head2 log MESSAGE
-
-Logs the message to standard error, using C<warn>.
+Defers to L<Net::IMAP::Server/log>.
 
 =cut
 
 sub log {
     my $self = shift;
-    my $msg  = shift;
-    chomp($msg);
-    warn $msg . "\n";
+    $self->server->log(@_);
 }
 
 =head2 untagged_response STRING
@@ -596,7 +603,7 @@ sub out {
     my $msg  = shift;
     if ( $self->io_handle and $self->io_handle->peerport ) {
         if ( $self->io_handle->print( $msg . "\r\n" ) ) {
-            $self->log(
+            $self->log( 4,
                 "S(@{[$self]},@{[$self->auth ? $self->auth->user : '???']},@{[$self->is_selected ? $self->selected->full_path : 'unselected']}): $msg"
             );
         } else {
